@@ -3,22 +3,20 @@ from unittest import TestCase
 
 import pytest
 
-from protego import Protego, _RuleSet
+from protego import Protego
+from protego._utils import _parse_time_period
 
 
 class TestProtego(TestCase):
     def test_allowed(self):
         content = (
-            "User-agent: * \n"
-            "Disallow: /disallowed \n"
-            "Allow: /allowed \n"
-            "Crawl-delay: 10"
+            "User-agent: * \nDisallow: /disallowed \nAllow: /allowed \nCrawl-delay: 10"
         )
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/allowed", "*"))
         self.assertFalse(rp.can_fetch("https://www.site.local/disallowed", "*"))
 
-        content = "User-agent: * \n" "Disallow: /d \n" "Crawl-delay: 10"
+        content = "User-agent: * \nDisallow: /d \nCrawl-delay: 10"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/abc/d", "*"))
         self.assertFalse(rp.can_fetch("https://www.site.local/disallowed", "*"))
@@ -42,105 +40,101 @@ class TestProtego(TestCase):
         self.assertFalse(rp.can_fetch("https://www.site.local/six", "*"))
 
     def test_length_based_precedence(self):
-        content = "User-agent: * \n" "Disallow: / \n" "Allow: /page"
+        content = "User-agent: * \nDisallow: / \nAllow: /page"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/page", "*"))
         self.assertFalse(rp.can_fetch("https://www.site.local/elsewhere", "*"))
 
-        content = "user-agent: FooBot\n" "disallow: /x/page.html\n" "allow: /x/\n"
+        content = "user-agent: FooBot\ndisallow: /x/page.html\nallow: /x/\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://foo.bar/x/page.html", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /x/page.html\n" "disallow: /x/\n"
+        content = "user-agent: FooBot\nallow: /x/page.html\ndisallow: /x/\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/x/page.html", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/x/", "FooBot"))
 
         # In case of equivalent disallow and allow patterns for the same
         # user-agent, allow is used.
-        content = "user-agent: FooBot\n" "disallow: \n" "allow: \n"
+        content = "user-agent: FooBot\ndisallow: \nallow: \n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/x/page.html", "FooBot"))
 
-        content = "user-agent: FooBot\n" "disallow: /\n" "allow: /\n"
+        content = "user-agent: FooBot\ndisallow: /\nallow: /\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/x/page.html", "FooBot"))
 
-        content = "user-agent: FooBot\n" "disallow: /x\n" "allow: /x/\n"
+        content = "user-agent: FooBot\ndisallow: /x\nallow: /x/\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/x/", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/x", "FooBot"))
 
-        content = (
-            "user-agent: FooBot\n" "disallow: /x/page.html\n" "allow: /x/page.html\n"
-        )
+        content = "user-agent: FooBot\ndisallow: /x/page.html\nallow: /x/page.html\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/x/page.html", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /page\n" "disallow: /*.html\n"
+        content = "user-agent: FooBot\nallow: /page\ndisallow: /*.html\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/page", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/page.html", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /x/page.\n" "disallow: /*.html\n"
+        content = "user-agent: FooBot\nallow: /x/page.\ndisallow: /*.html\n"
         rp = Protego.parse(content=content)
         # Longest match wins.
         self.assertTrue(rp.can_fetch("http://foo.bar/x/page.html", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/x/y.html", "FooBot"))
 
-        content = (
-            "User-agent: *\n" "Disallow: /x/\n" "User-agent: FooBot\n" "Disallow: /y/\n"
-        )
+        content = "User-agent: *\nDisallow: /x/\nUser-agent: FooBot\nDisallow: /y/\n"
         rp = Protego.parse(content=content)
         # Most specific group for FooBot allows implicitly /x/page.
         self.assertTrue(rp.can_fetch("http://foo.bar/x/page", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/y/page", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /p\n" "disallow: /\n"
+        content = "user-agent: FooBot\nallow: /p\ndisallow: /\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://example.com/page", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /folder\n" "disallow: /folder\n"
+        content = "user-agent: FooBot\nallow: /folder\ndisallow: /folder\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://example.com/folder/page", "FooBot"))
 
-        content = "user-agent: FooBot\n" "disallow: /folder\n" "allow: /folder\n"
+        content = "user-agent: FooBot\ndisallow: /folder\nallow: /folder\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://example.com/folder/page", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /page\n" "disallow: /*.htm\n"
+        content = "user-agent: FooBot\nallow: /page\ndisallow: /*.htm\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://example.com/page.htm", "FooBot"))
 
-        content = "user-agent: FooBot\n" "allow: /$\n" "disallow: /\n"
+        content = "user-agent: FooBot\nallow: /$\ndisallow: /\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://example.com/", "FooBot"))
         self.assertFalse(rp.can_fetch("http://example.com/page.html", "FooBot"))
 
     def test_escaped_url(self):
-        content = "User-agent: * \n" "Disallow: / \n" "Allow: /a%3cd.html"
+        content = "User-agent: * \nDisallow: / \nAllow: /a%3cd.html"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/a<d.html", "*"))
         self.assertTrue(rp.can_fetch("https://www.site.local/a%3cd.html", "*"))
 
-        content = "User-agent: * \n" "Disallow: / \n" "Allow: /a%3c*"
+        content = "User-agent: * \nDisallow: / \nAllow: /a%3c*"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/a<d.html", "*"))
         self.assertTrue(rp.can_fetch("https://www.site.local/a%3cd.html", "*"))
 
     def test_unescaped_url(self):
-        content = "User-agent: * \n" "Disallow: / \n" "Allow: /a<d.html"
+        content = "User-agent: * \nDisallow: / \nAllow: /a<d.html"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/a<d.html", "*"))
         self.assertTrue(rp.can_fetch("https://www.site.local/a%3cd.html", "*"))
 
-        content = "User-agent: * \n" "Disallow: / \n" "Allow: /a<*"
+        content = "User-agent: * \nDisallow: / \nAllow: /a<*"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("https://www.site.local/a<d.html", "*"))
         self.assertTrue(rp.can_fetch("https://www.site.local/a%3cd.html", "*"))
 
     def test_url_parts(self):
-        content = "User-agent: * \n" "Disallow: /path;params?query \n"
+        content = "User-agent: * \nDisallow: /path;params?query \n"
         rp = Protego.parse(content=content)
         self.assertFalse(
             rp.can_fetch(
@@ -148,12 +142,12 @@ class TestProtego(TestCase):
             )
         )
 
-        content = "User-agent: * \n" "Disallow: /? \n"
+        content = "User-agent: * \nDisallow: /? \n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("/?query", "*"))
         self.assertTrue(rp.can_fetch("/", "*"))
 
-        content = "User-agent: * \n" "Disallow: /; \n"
+        content = "User-agent: * \nDisallow: /; \n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("/;params", "*"))
         self.assertTrue(rp.can_fetch("/", "*"))
@@ -176,20 +170,14 @@ class TestProtego(TestCase):
 
     def test_no_sitemaps(self):
         content = (
-            "User-agent: * \n"
-            "Disallow: /disallowed \n"
-            "Allow: /allowed \n"
-            "Crawl-delay: 10"
+            "User-agent: * \nDisallow: /disallowed \nAllow: /allowed \nCrawl-delay: 10"
         )
         rp = Protego.parse(content=content)
         self.assertTrue(not list(rp.sitemaps))
 
     def test_no_preferred_host(self):
         content = (
-            "User-agent: * \n"
-            "Disallow: /disallowed \n"
-            "Allow: /allowed \n"
-            "Crawl-delay: 10"
+            "User-agent: * \nDisallow: /disallowed \nAllow: /allowed \nCrawl-delay: 10"
         )
         rp = Protego.parse(content=content)
         self.assertTrue(rp.preferred_host is None)
@@ -218,7 +206,7 @@ class TestProtego(TestCase):
         self.assertTrue(rp.crawl_delay("*") is None)
 
     def test_no_crawl_delay(self):
-        content = "User-agent: * \n" "Disallow: /disallowed \n" "Allow: /allowed"
+        content = "User-agent: * \nDisallow: /disallowed \nAllow: /allowed"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.crawl_delay("*") is None)
 
@@ -238,30 +226,37 @@ class TestProtego(TestCase):
         rp = Protego.parse(content=content)
 
         req_rate = rp.request_rate("one")
+        assert req_rate is not None
         self.assertTrue(req_rate.requests == 1)
         self.assertTrue(req_rate.seconds == 10)
         self.assertTrue(req_rate.start_time is None)
         self.assertTrue(req_rate.end_time is None)
 
         req_rate = rp.request_rate("two")
+        assert req_rate is not None
         self.assertTrue(req_rate.requests == 100)
         self.assertTrue(req_rate.seconds == 900)
         self.assertTrue(req_rate.start_time is None)
         self.assertTrue(req_rate.end_time is None)
 
         req_rate = rp.request_rate("three")
+        assert req_rate is not None
         self.assertTrue(req_rate.requests == 400)
         self.assertTrue(req_rate.seconds == 3600)
         self.assertTrue(req_rate.start_time is None)
         self.assertTrue(req_rate.end_time is None)
 
         req_rate = rp.request_rate("four")
+        assert req_rate is not None
         self.assertTrue(req_rate.requests == 9000)
         self.assertTrue(req_rate.seconds == 86400)
         self.assertTrue(req_rate.start_time is None)
         self.assertTrue(req_rate.end_time is None)
 
         req_rate = rp.request_rate("five")
+        assert req_rate is not None
+        assert req_rate.start_time is not None
+        assert req_rate.end_time is not None
         self.assertTrue(req_rate.requests == 1)
         self.assertTrue(req_rate.seconds == 10)
         self.assertTrue(req_rate.start_time.hour == 18)
@@ -397,16 +392,14 @@ class TestProtego(TestCase):
         self.assertTrue(rp.can_fetch("xyz/foo.js", "Rule7TestBot"))
         self.assertTrue(rp.can_fetch("/inlife/daily/fashion-20160727/", "Rule7TestBot"))
 
-        content = (
-            "User-agent: FooBot\n" "Disallow: /foo/bar/quz\n" "Allow: /foo/*/qux\n"
-        )
+        content = "User-agent: FooBot\nDisallow: /foo/bar/quz\nAllow: /foo/*/qux\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://foo.bar/foo/bar/quz", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/quz", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo//quz", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bax/quz", "FooBot"))
 
-        content = "User-agent: FooBot\n" "Disallow: /foo/bar$\n" "Allow: /foo/bar/qux\n"
+        content = "User-agent: FooBot\nDisallow: /foo/bar$\nAllow: /foo/bar/qux\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://foo.bar/foo/bar", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/qux", "FooBot"))
@@ -591,10 +584,7 @@ class TestProtego(TestCase):
         self.assertTrue(rp.can_fetch("https://site.local/default-ua", "two"))
 
         content = (
-            "User-agent: FooBot\n"
-            "# Disallow: /\n"
-            "Disallow: /foo/quz#qux\n"
-            "Allow: /\n"
+            "User-agent: FooBot\n# Disallow: /\nDisallow: /foo/quz#qux\nAllow: /\n"
         )
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar", "FooBot"))
@@ -902,6 +892,9 @@ class TestProtego(TestCase):
         self.assertFalse(rp.can_fetch("http://foo.bar/horcrux", "harry potter"))
         self.assertTrue(rp.can_fetch("http://foo.bar/abc", "harry potter"))
         req_rate = rp.request_rate("harry potter")
+        assert req_rate is not None
+        assert req_rate.start_time is not None
+        assert req_rate.end_time is not None
         self.assertTrue(req_rate.requests == 1)
         self.assertTrue(req_rate.seconds == 10)
         self.assertTrue(req_rate.start_time.hour == 18)
@@ -952,39 +945,19 @@ class TestProtego(TestCase):
         )
 
         unix_file = (
-            "User-Agent: foo\n"
-            "Allow: /some/path\n"
-            "User-Agent: bar\n"
-            "\n"
-            "\n"
-            "Disallow: /\n"
+            "User-Agent: foo\nAllow: /some/path\nUser-Agent: bar\n\n\nDisallow: /\n"
         )
 
         mac_file = (
-            "User-Agent: foo\r"
-            "Allow: /some/path\r"
-            "User-Agent: bar\r"
-            "\r"
-            "\r"
-            "Disallow: /\r"
+            "User-Agent: foo\rAllow: /some/path\rUser-Agent: bar\r\r\rDisallow: /\r"
         )
 
         no_final_line_ending = (
-            "User-Agent: foo\n"
-            "Allow: /some/path\n"
-            "User-Agent: bar\n"
-            "\n"
-            "\n"
-            "Disallow: /"
+            "User-Agent: foo\nAllow: /some/path\nUser-Agent: bar\n\n\nDisallow: /"
         )
 
         mixed_file = (
-            "User-Agent: foo\n"
-            "Allow: /some/path\r\n"
-            "User-Agent: bar\n"
-            "\r\n"
-            "\n"
-            "Disallow: /"
+            "User-Agent: foo\nAllow: /some/path\r\nUser-Agent: bar\n\r\n\nDisallow: /"
         )
 
         test_url = "http://site.local/some/path/"
@@ -1002,7 +975,7 @@ class TestProtego(TestCase):
             self.assertFalse(rp.can_fetch(test_url, "bar"))
 
     def test_index_html_is_directory(self):
-        content = "User-Agent: *\n" "Allow: /allowed-slash/index.html\n" "Disallow: /\n"
+        content = "User-Agent: *\nAllow: /allowed-slash/index.html\nDisallow: /\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.com/allowed-slash/", "footbot"))
         self.assertTrue(
@@ -1026,27 +999,27 @@ class TestProtego(TestCase):
             )
         )
 
-        content = "User-agent: FooBot\n" "Disallow: /\n" "Allow: /foo/bar/ツ\n"
+        content = "User-agent: FooBot\nDisallow: /\nAllow: /foo/bar/ツ\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/%E3%83%84", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/ツ", "FooBot"))
 
-        content = "User-agent: FooBot\n" "Disallow: /\n" "Allow: /foo/bar/%E3%83%84\n"
+        content = "User-agent: FooBot\nDisallow: /\nAllow: /foo/bar/%E3%83%84\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/%E3%83%84", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/ツ", "FooBot"))
 
-        content = "User-agent: FooBot\n" "Disallow: /\n" "Allow: /foo/bar/%62%61%7A\n"
+        content = "User-agent: FooBot\nDisallow: /\nAllow: /foo/bar/%62%61%7A\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/baz", "FooBot"))
         self.assertTrue(rp.can_fetch("http://foo.bar/foo/bar/%62%61%7A", "FooBot"))
 
     def test_url_case_sensitivity(self):
-        content = "user-agent: FooBot\n" "disallow: /x/\n"
+        content = "user-agent: FooBot\ndisallow: /x/\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://foo.bar/x/y", "FooBot"))
 
-        content = "user-agent: FooBot\n" "disallow: /X/\n"
+        content = "user-agent: FooBot\ndisallow: /X/\n"
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/x/y", "FooBot"))
 
@@ -1071,9 +1044,7 @@ class TestProtego(TestCase):
 
     def test_escaped_special_symbols(self):
         """Percent encoded special symbols should be treated as ordinary characters."""
-        content = (
-            "user-agent: FooBot\n" "disallow: /x/abc%24\n" "disallow: /x%2Ax/abc\n"
-        )
+        content = "user-agent: FooBot\ndisallow: /x/abc%24\ndisallow: /x%2Ax/abc\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://foo.bar/x/abc$abc", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/x/abc$", "FooBot"))
@@ -1084,14 +1055,14 @@ class TestProtego(TestCase):
 
     def test_special_symbols_dual_behaviour(self):
         """Special symbols such as * and $, should also be treated as an ordinary character"""
-        content = "user-agent: FooBot\n" "disallow: /x/abc$\n" "disallow: /x*x/abc\n"
+        content = "user-agent: FooBot\ndisallow: /x/abc$\ndisallow: /x*x/abc\n"
         rp = Protego.parse(content=content)
         self.assertFalse(rp.can_fetch("http://foo.bar/x*x/abc", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/x/abc$", "FooBot"))
         self.assertFalse(rp.can_fetch("http://foo.bar/x/abc%24", "FooBot"))
 
     def test_with_absolute_urls(self):
-        content = "user-agent: *\n" "disallow: http://ms-web00.walkerplus.com/\n"
+        content = "user-agent: *\ndisallow: http://ms-web00.walkerplus.com/\n"
 
         rp = Protego.parse(content=content)
         self.assertTrue(rp.can_fetch("http://foo.bar/", "FooBot"))
@@ -1118,7 +1089,7 @@ class TestProtego(TestCase):
     def test_bytestrings(self):
         content = b"User-Agent: FootBot\nDisallow: /something"
         with self.assertRaises(ValueError) as context:
-            Protego.parse(content=content)
+            Protego.parse(content=content)  # type: ignore[arg-type]
 
         self.assertEqual("Protego.parse expects str, got bytes", str(context.exception))
 
@@ -1137,17 +1108,17 @@ class TestProtego(TestCase):
         content = "User-Agent: *\nVisit-time: 0200 0630\nUser-Agent: NoTime"
         rp = Protego.parse(content)
         visit_time = rp.visit_time("FooBoot")
+        assert visit_time is not None
         self.assertEqual(visit_time.start_time, time(2, 0))
         self.assertEqual(visit_time.end_time, time(6, 30))
         self.assertIsNone(rp.visit_time("NoTime"))
 
     def test_parse_time_period(self):
-        rs = _RuleSet(None)
-        start_time, end_time = rs._parse_time_period("0100-1000")
+        start_time, end_time = _parse_time_period("0100-1000")
         self.assertEqual(start_time, time(1, 0))
         self.assertEqual(end_time, time(10, 0))
 
-        start_time, end_time = rs._parse_time_period("0500 0600", separator=" ")
+        start_time, end_time = _parse_time_period("0500 0600", separator=" ")
         self.assertEqual(start_time, time(5, 0))
         self.assertEqual(end_time, time(6, 0))
 
@@ -1167,6 +1138,6 @@ class TestProtego(TestCase):
     ],
 )
 def test_leading_asterisk(allow, disallow, url, allowed):
-    content = f"User-Agent: *\n" f"allow: {allow}\n" f"disallow: {disallow}\n"
+    content = f"User-Agent: *\nallow: {allow}\ndisallow: {disallow}\n"
     rp = Protego.parse(content)
     assert rp.can_fetch(url, "*") == allowed
